# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find FPs (Function Point) metrics """


from baseblock import BaseObject


class MetricFpCounter(BaseObject):
    """ Metrics to Find FPs (Function Point) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_lines: list) -> dict:

        input_lines = [x.strip() for x in input_lines if x and len(x)]
        input_lines = [x for x in input_lines if x.startswith('def ')]

        return len(input_lines)



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find TLoC (Total-Lines-of-Code) metrics """


from baseblock import FileIO
from baseblock import Enforcer
from baseblock import BaseObject


class MetricTLocCounter(BaseObject):
    """ Metrics to Find TLoC (Total-Lines-of-Code) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_lines: list) -> dict:

        input_lines = [x for x in input_lines if x and len(x)]
        input_lines = [x for x in input_lines if not x.strip().startswith('#')]

        line_no = 0
        normalized = []
        
        flag_comment = False
        total_lines = len(input_lines)

        while line_no < total_lines:

            line = input_lines[line_no]

            if line.strip().startswith('"""'):
                flag_comment = not flag_comment

            if not flag_comment:
                normalized.append(line)

            line_no += 1

        return len(normalized)



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Metrics to Find TLoC (Total-Lines-of-Code) metrics """


from baseblock import BaseObject


class MetricImportCounter(BaseObject):
    """ Metrics to Find TLoC (Total-Lines-of-Code) metrics """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                input_lines: list) -> dict:

        def is_import(input_line: str) -> bool:
            if input_line.startswith('from ') and ' import ' in input_line:
                return True
            if input_line.startswith('input '):
                return True
            return False

        input_lines = [x.strip() for x in input_lines if x and len(x)]
        input_lines = [x for x in input_lines if is_import(x)]

        return len(input_lines)



# !/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Micro Metrics Orchestrator """


from baseblock import FileIO
from baseblock import Enforcer
from baseblock import BaseObject

from micro_metrics.svc import FindFileMetrics
from micro_metrics.svc import ReadFileContent
from micro_metrics.svc import FindFolderMetrics
from micro_metrics.svc import ComputeComplexity


class Orchestrator(BaseObject):
    """ Micro Metrics Orchestrator """

    def __init__(self):
        """ Change Log

        Created:
            5-Sept-2022
            craigtrim@gmail.com
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _sum_file_metrics(results: list) -> dict:

        loc = 0
        tloc = 0
        ccs = 0
        fps = 0
        imports = 0

        for result in results:
            loc += result['LOC']
            tloc += result['TLOC']
            ccs += result['CC']
            fps += result['FP']
            imports += result['Imports']

        return {
            'LOC': loc,
            'TLOC': tloc,
            'CC': ccs,
            'FP': fps,
            'Imports': imports
        }

    def run(self,
            microservice_directory: str) -> dict:

        if self.isEnabledForDebug:
            Enforcer.is_str(microservice_directory)

        FileIO.exists_or_error(microservice_directory)

        project_files = FileIO.load_all_files(microservice_directory, exclude=[
            '.git', '.pytest_cache', '__pycache__'])

        d_py_files = ReadFileContent().process(project_files['py'])
        if self.isEnabledForDebug:
            Enforcer.is_dict(d_py_files)

        results = FindFileMetrics().process(d_py_files)

        d_sum_file = self._sum_file_metrics(results)
        d_sum_folder = FindFolderMetrics().process(microservice_directory)

        d_sum_file.update(d_sum_folder)
        complexity_score = ComputeComplexity().process(d_sum_file)

        d_sum_file['Complexity'] = complexity_score
        
        return d_sum_file



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['micro_metrics', 'micro_metrics.bp', 'micro_metrics.dmo', 'micro_metrics.svc']

package_data = \
{'': ['*']}

install_requires = \
['baseblock']

setup_kwargs = {
    'name': 'micro-metrics',
    'version': '0.1.2',
    'description': 'Code Metrics on Python Microservices',
    'long_description': None,
    'author': 'Craig Trim',
    'author_email': 'craigtrim@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.8.5',
}


setup(**setup_kwargs)
